/*
    $Id: configfile.cpp,v 1.1.1.1 2001/08/21 10:04:43 samwise97 Exp $

    configfile.cpp -- simple class for reading/writing a configuration file
    Copyright (C) 2001 Iwan Mouwen <iwan@vvier.tudelft.nl>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "configfile.h"
#include <iostream.h> //cout
#include <qregexp.h>


#define __DEBUG(c) c

Item::Item( const QString &line )
{
	iskeyvalue = false;
        this->line = new QString( line );
	this->key = 0;
	this->value = 0;
}

Item::Item( const QString &key, const QString &value )
{
	iskeyvalue = true;
	this->key = new QString( key );
	this->value = new QString( value );
	this->line = 0;
}

Item::~Item()
{       	
	if ( line ) delete line;
	if ( key ) delete key;
	if ( value ) delete value;
}

bool Item::isKey( const QString &key )
{
	if ( iskeyvalue )
		if ( *this->key == key )
			return true;	

	return false;
}

void Item::setValue( const QString &value )
{
	if (!this->value) this->value = new QString;
	*this->value = value;
}

QString Item::getString()
{
	return *value;
}

Section::Section( const QString &section )
{
	this->name = section;
	itemList.setAutoDelete( true );
}

Section::~Section()
{
	Clear();
}

void Section::Clear()
{
	itemList.clear();
}

int Section::getCount()
{
	return itemList.count();	
}

QString Section::getName()
{
	return name;	
}

bool Section::isSection( const QString &name )
{
	return ( this->name == name );
}

bool Section::findItem( const QString &key, Item **theitem )
{
	Item * i;
	for( i = itemList.first(); i != 0; i = itemList.next() ) {
		if( i->isKey( key ) ) {
			*theitem = i;
			return true;
		}
	}
	return false;	
}

bool Section::isKeyValue( const QString &line, QString &key, QString &value )
{
	int i;
	i = line.find( "=" );

	if ( i > 0 ) {
		key = line.mid( 0, i );
		value = line.mid( i + 1 );
		key = key.stripWhiteSpace();
		value = value.stripWhiteSpace();
		return true;
	}
	return false;
}

void Section::addString( const QString &key, const QString &value )
{
	Item * i;
	if ( findItem( key, &i ) ) {
		i->setValue( value );
	} else {
		i = new Item( key, value );
		itemList.append( i );
	}
}

void Section::addLine( const QString &line )
{
	QString key, value;
	Item * i = 0;
	if (isKeyValue( line, key, value ))
		addString( key, value );
	else
	if ( !line.isEmpty() )
		i = new Item( line );

	if ( i )
		itemList.append( i );
}

QList<Item> * Section::getList()
{
	Item * i;
	QList<Item> * list = new QList<Item>;
	list->setAutoDelete( true );
	for( i = itemList.first(); i != 0; i = itemList.next() ) {
		if( i->isKeyValue() ) {
			Item * j = new Item(i->getKey(), i->getValue());
			list->append(j);
		}
	}
	return list;
}

ConfigFile::ConfigFile( const QString &fileName )
{
	this->fileName = fileName;
	sectionList.setAutoDelete( true );
	Init();
	readFile();
}

ConfigFile::~ConfigFile()
{
}

void ConfigFile::Clear()
{
	sectionList.clear();
}

void ConfigFile::Init()
{
	Clear();

	// empty section at begin of the file.
	currentSection = new Section( "" );
	sectionList.append( currentSection );
}

Section* ConfigFile::addSection( const QString &section )
{
	Section* newSection = new Section( section );
	sectionList.append(newSection);
	//currentSection = newSection;
	return newSection;
}

bool ConfigFile::findSection( const QString &section, Section **thesection, bool create=true )
{
	Section * s;
	for( s = sectionList.first(); s != 0; s = sectionList.next() ) {
		if( s->isSection( section ) ) {
			*thesection = s;
			return true;
		}
	}
	// didn't find it.
	if (create) {
		*thesection = addSection( section );
		return true;
	} else {
		return false;
	}	
}

bool ConfigFile::isSectionIdentifier( const QString &line, QString &name )
{
	if (! line.contains( QRegExp( "^\\s*\\[.+\\]\\s*$" ) ) ) return false;
	int i, j;
	i = line.find( "[" );
	j = line.findRev( "]" );

	if ( j - i > 1 ) {
		name = line.mid( i + 1, j - i - 1 );
		return true;
	}
	return false;
}

void ConfigFile::addLine( const QString &line )
{
	QString name;
	if( isSectionIdentifier( line, name ) ) {
		Section * s;
		if ( findSection( name, &s, true ) ) {
                 	currentSection = s;
		}
	} else {
		currentSection->addLine( line );		
	}
}

QString ConfigFile::getString( const QString &section, const QString &key, const QString &Default )
{
	Section * s;
	if ( findSection( section, &s, false ) ) {
		Item * i;
		if ( s->findItem( key, &i ) ) {
                	return i->getString();
		}
	}		
	return Default;         	
}

QList<Item> * ConfigFile::getList( const QString &section )
{
	Section * s;
	if ( findSection( section, &s, false ) ) {
		return s->getList();		
	} else
		return new QList<Item>;
}


/*
void ConfigFile::addString( const QString &section, const QString &key, const QString &value )
{
	Section * s;
	if ( findSection( section, *s, true ) ) {
		s->addString(key, value);
	}
}
*/

void ConfigFile::readFile()
{
	QFile f( fileName );
	if (f.open ( IO_ReadOnly )) {
		QTextStream t( &f );
		QString s;
		while ( !t.eof() ) {
			s = t.readLine();
			addLine( s );
		}
	}
	f.close();
}
