/*
    $Id: limage.cpp,v 1.1.1.1 2001/08/21 10:04:43 samwise97 Exp $

    image.cpp -- implementation of Image class
    Copyright (C) 2001 Iwan Mouwen <iwan@vvier.tudelft.nl>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "limage.h"
#include <qimage.h>
#include <qpainter.h>
#include <qpixmap.h>


#define FIXEDWIDTH 250
#define FIXEDHEIGHT 250

LImage::LImage( bool fixed, QWidget *parent, const char *name )
        : QWidget ( parent, name )
{
        pm = new QPixmap;
		
	if (fixed) {
		setMinimumSize( FIXEDWIDTH, FIXEDHEIGHT );
		setMaximumSize( FIXEDWIDTH, FIXEDHEIGHT );
		setSizePolicy ( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
		ftheimage = 0;
	} else {
		setMinimumSize( 10, 10 );
		setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );
	        ftheimage = 0;
	}

	ffixed=fixed;
	fx=-1;
	fy=-1;
	fsister=0;
        Xscale=0;
	Yscale=0;
	Offset=new QPoint(0, 0);

	//TODO: should I set other flags?
        setWFlags(WResizeNoErase);

}

LImage::~LImage()
{
	if ( ffixed ) {
		delete ftheimage;
	}
}

void LImage::setImage( QImage *image )
{
	if ( ffixed ) {
		if (!ftheimage) {
			ftheimage = new QImage( FIXEDWIDTH, FIXEDHEIGHT, 8 );
		}
		//TODO: keep aspect ratio
		*ftheimage = image->smoothScale( FIXEDWIDTH, FIXEDHEIGHT );
		Xscale = ( 100 * image->width() / FIXEDWIDTH );
		Yscale = ( 100 * image->height() / FIXEDHEIGHT );
	} else {
		ftheimage = image;
	}
	repaint( rect() );
}

void LImage::setSister ( LImage *sister )
{
	fsister=sister;
}

QPoint LImage::getOffset()
{
	if ( ffixed ) {
		return QPoint( 0, 0 );
	} else {
		return *Offset;
	}
}

QPoint LImage::calculateGlobalXY( QPoint p )
{
// TODO add a threshold
	QPoint gp;
	if ( ffixed ) {
		gp.setX( p.x() * Xscale / 100 );
		gp.setY( p.y() * Yscale / 100 );
	} else {
		gp.setX( p.x() + Offset->x() );
		gp.setY( p.y() + Offset->y() );
	}

	return gp;
}

void LImage::gotoXY( QPoint p, bool tell_sister=true )
{
	/* a fixed LImage gets the offset from her sister in draw() */
	if ( !ffixed ) {
		Offset->setX( p.x() - width() / 2 );
		Offset->setY( p.y() - height() / 2 );
	}

	if ( tell_sister && fsister ) {
		fsister->gotoXY( p, false );
	}
        repaint ( false );
}

void LImage::gotoXY( int x, int y, bool tell_sister=true )
{
	gotoXY( QPoint(x, y), tell_sister );
}

void LImage::mousePressEvent( QMouseEvent *e )
{
    if ( e->button() != LeftButton )
	return;
}

void LImage::mouseReleaseEvent( QMouseEvent *e )
{
	if ( e->button() == LeftButton && ftheimage ) {
		QPoint p = calculateGlobalXY( e->pos() );
		gotoXY( p );
	}
}


/*
  definitions for a scalable LImage:
  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 |\_                                                       ftheimage |
    \_ Offset
 |    \_                                                             |
        \            globalRect = QRect ( Offset, LImage::size() )
 |       +++++++++++++++++++                                         |
         +                 +
 |       +                 +                                         |
         +                 +
 |       +                 +                                         |
         +                 +
 |       +++++++++++++++++++                                         |

 |                                                                   |

 |                                                                   |
  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/

//#define extract_rect(r) (r)->left(), (r)->top(), (r)->right(), (r)->bottom()
void LImage::draw( QRect r )
{
	if ( !ftheimage ) { return; }
	if ( ffixed ) {
		QPainter p( pm );
		p.drawImage( QPoint(0, 0), *ftheimage );

		QPoint O = fsister->getOffset();
		int x, y, w, h;
		x = O.x() * 100 / Xscale;
		y = O.y() * 100 / Yscale;
		w = fsister->width() * 100 / Xscale;
		h = fsister->height() * 100 / Yscale;
		
		bitBlt( pm, x, y, pm, x, y, w, h, NotROP );
	} else {
               	QPainter p( pm );
		QRect globalRect = QRect( *Offset, size());
		p.drawImage ( QPoint(0, 0), *ftheimage, globalRect );
		p.setBrush( QBrush( QColor( 90, 120, 80 ) ) );
		p.setPen ( NoPen );

		if ( Offset->x() < 0 ) {
			QRect tmpRect = rect();
			tmpRect.setRight( -Offset->x() );
			p.drawRect( tmpRect );
		}
		if ( Offset->y() < 0 ) {
			QRect tmpRect = rect();
			tmpRect.setBottom( -Offset->y() );
			p.drawRect( tmpRect );
		}
		if ( Offset->x() + width() > ftheimage->width() ) {
                 	QRect tmpRect = rect();
			tmpRect.setLeft( ftheimage->width() - Offset->x() );
			p.drawRect( tmpRect );
		}
		if ( Offset->y() + height() > ftheimage->height() ) {
			QRect tmpRect = rect();
			tmpRect.setTop( ftheimage->height() - Offset->y() );
			p.drawRect( tmpRect );
		}
		int midX = width() / 2;
		int midY = height() / 2;
		QPen pn = QPen( QColor( 255, 255, 255 ), 1, SolidLine );
		p.setPen( pn );

		/* diagonal */
		//p.drawLine( midX - 10, midY - 10, midX + 10, midY + 10 );
		//p.drawLine( midX - 10, midY + 10, midX + 10, midY - 10 );

		/* straight */
		p.drawLine( midX - 10, midY, midX + 10, midY );
		p.drawLine( midX, midY + 10, midX, midY - 10 );

		
	}
	QPainter p( this );
	p.drawPixmap( QPoint( r.x(), r.y() ), *pm, r );
}

void LImage::paintEvent( QPaintEvent *e )
{
	draw( e->rect() );
}

void LImage::resizeEvent ( QResizeEvent *e )
{
	pm->resize( e->size() );
	if ( !ffixed ) { fsister->draw( fsister->rect() ); }
}
